-- Listing 6.7
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;


ENTITY period_counter IS
  PORT(clk, reset           : IN  std_logic;
           start, si        : IN  std_logic;
           ready, done_tick : OUT std_logic;
           prd              : OUT std_logic_vector(31 DOWNTO 0));  -- was  (9 DOWNTO 0)
END period_counter;



ARCHITECTURE arch OF period_counter IS


--mostafa
attribute KEEP : string;
attribute S    : string;
--mostafa



  CONSTANT CLK_MS_COUNT        : integer := 5;  -- 50000; -- 1 ms tick
  TYPE state_type IS (idle, waite, count, done);
  SIGNAL state_reg, state_next : state_type;
  SIGNAL t_reg, t_next         : unsigned(15 DOWNTO 0);  -- up to 50000
  SIGNAL p_reg, p_next         : unsigned(31 DOWNTO 0);   -- up to 1 sec    -- was  unsigned(9 DOWNTO 0);
  SIGNAL delay_reg             : std_logic;
  SIGNAL edge                  : std_logic;
  
  
  
  attribute KEEP of state_reg   : signal is "TRUE";
  attribute S    of state_reg   : signal is "TRUE";
  attribute KEEP of state_next  : signal is "TRUE";
  attribute S    of state_next  : signal is "TRUE";
  attribute KEEP of t_reg       : signal is "TRUE";
  attribute S    of t_reg       : signal is "TRUE";
  attribute KEEP of t_next      : signal is "TRUE";
  attribute S    of t_next      : signal is "TRUE";
  attribute KEEP of p_reg       : signal is "TRUE";
  attribute S    of p_reg       : signal is "TRUE";
  attribute KEEP of p_next      : signal is "TRUE";
  attribute S    of p_next      : signal is "TRUE";
  attribute KEEP of delay_reg   : signal is "TRUE";
  attribute S    of delay_reg   : signal is "TRUE";
  attribute KEEP of edge        : signal is "TRUE";
  attribute S    of edge        : signal is "TRUE";    
  
  
  
  
  
  
BEGIN
  -- state and data register
  PROCESS(clk, reset)
  BEGIN
    IF reset = '1' THEN
      state_reg <= idle;
      t_reg     <= (OTHERS => '0');
      p_reg     <= (OTHERS => '0');
      delay_reg <= '0';
    ELSIF (clk'event AND clk = '1') THEN
      state_reg <= state_next;
      t_reg     <= t_next;
      p_reg     <= p_next;
      delay_reg <= si;
    END IF;
  END PROCESS;

  edge <= (NOT delay_reg) AND si;

  PROCESS(start, edge, state_reg, t_reg, t_next, p_reg)
  BEGIN
    ready      <= '0';
    done_tick  <= '0';
    state_next <= state_reg;
    p_next     <= p_reg;
    t_next     <= t_reg;
    
	 CASE state_reg IS
      WHEN idle =>
        ready <= '1';
        IF (start = '1') THEN
          state_next <= waite;
        END IF;
      WHEN waite =>                       -- wait for the first edge
        IF (edge = '1') THEN
          state_next <= count;
          t_next     <= (OTHERS => '0');
          p_next     <= (OTHERS => '0');
			 ----- added by me
			 if (p_reg = "00000101111101011110000011110110") then
						p_next <= (others => '0');
						t_next <= (others => '0');
			 end if;
			 ------ end of added by me
        END IF;
      WHEN count =>
        IF (edge = '1') THEN              -- 2nd edge arrived
          state_next <= done;
        ELSE                              -- otherwise count
          IF t_reg = CLK_MS_COUNT-1 THEN  -- 1ms tick
            t_next <= (OTHERS => '0');
            p_next <= p_reg + 1;
          ELSE
            t_next <= t_reg + 1;
          END IF;
        END IF;
      WHEN done =>
        done_tick  <= '1';
        state_next <= idle;
    END CASE;
  END PROCESS;
  prd <= std_logic_vector(p_reg);
END arch;
